/* FILE: tkshell.cc                 -*-Mode: c++-*-
 *
 * Sample code to bring up a Tcl/Tk shell.
 *
 */

/*
 * Compile+Link with:
 *
 *    gcc -I/usr/include/tcl tkshell.cc \
 *        -L/usr/X11/lib -ltk -ltcl -lX11 -lm -o tkshell
 *
 */

#include <tcl.h>
#include <tk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define SCRATCH_BUF_SIZE 1024
static char buf[SCRATCH_BUF_SIZE];

int SayHello(ClientData,Tcl_Interp *interp,int argc,char** argv)
{
  Tcl_ResetResult(interp);
  if(argc!=2) {
    sprintf(buf,"SayHello must be called with"
	    " 1 argument: Person to greet"
	    " (%d arguments passed)",argc-1);
    Tcl_AppendResult(interp,buf,(char *)NULL);
    return TCL_ERROR;
  }

  sprintf(buf,"puts \"Hello %s.\"",argv[1]);

  return Tcl_Eval(interp,buf);
}

int RegisterTclCommand(Tcl_Interp* interp,
			const char* name,Tcl_CmdProc* cmd)
{
  // I don't think it is necessary to copy name to a writeable
  // buffer, but better safe than sorry.
  strncpy(buf,name,sizeof(buf));  buf[sizeof(buf)-1]='\0';
  if(Tcl_CreateCommand(interp,buf,cmd,NULL,NULL)==NULL) {
    fprintf(stderr,"Unable to register command ->%s<- "
	    "with Tcl Interpreter\n",buf);
    fflush(stderr);
    exit(1);
  }
  return TCL_OK;
}

int Tcl_AppInit(Tcl_Interp* interp)
{
  // Initialize tcl and tk interpreters
  if(Tcl_Init(interp) != TCL_OK) {
    fprintf(stderr,"Tcl_Init error: %s\n",interp->result); fflush(stderr);
    return TCL_ERROR;
  }
  if(Tk_Init(interp)  != TCL_OK) {
    fprintf(stderr,"Tk_Init error: %s\n",interp->result); fflush(stderr);
    return TCL_ERROR;
  }

  // Register C/C++ routines with interpreter
  RegisterTclCommand(interp,"SayHello",SayHello);

  return TCL_OK;
}

int main(int argc,char **argv)
{

  argc=1; /* Tk_Main will try to input command line arg's as input files */
  Tk_Main(argc,argv,Tcl_AppInit);

  return 0;
}
